#!/bin/bash

prepd_pid=0
turbod_pid=0

function cleanup()
{
	set +e  # tolerate errors during cleanup 
	local exit_code=$?

	if [[ $predp_pid == 0 ]]; then
		exit_code=1
	else
		kill -9 $prepd_pid
		wait $prepd_pid
		if [[ $? != 137 ]] ; then
			exit_code=1
		fi

	fi
	if [[ $turbod_pid == 0 ]]; then
		exit_code=1
	else
		kill -9 $turbod_pid
		wait $turbod_pid
		if [[ $? != 137 ]] ; then
			exit_code=1
		fi

	fi
	dropdb turbodb || true
	exit $exit_code
}

function setup_server()
{
	dropdb turbodb || true # just to make sure it's a clean test
	$TURBO_HOME/bin/turbod.exe &
	turbod_pid=$!

	sleep 5

	$TURBO_HOME/bin/prepd.exe &
	prepd_pid=$!


}

function test1()
{
	# create boost, add version and list versions
	local bid=$($TURBO_HOME/bin/turbo-cli.exe boost add -q ls )
	echo "Boost id is '$bid'"
	local vid=$($TURBO_HOME/bin/turbo-cli.exe version add -q $bid $(which ls))
	echo "Version id is '$vid'"
	$TURBO_HOME/bin/turbo-cli.exe version list $vid
	return 0
}

function test2()
{
	# create boost, list versions when no versions exist
	local bid=$($TURBO_HOME/bin/turbo-cli.exe boost add -q ls2 )
	echo "Boost id is '$bid'"
	! $TURBO_HOME/bin/turbo-cli.exe version list $bid # expected fail of command, so negate to ensure it fails or we cleanup/exit.
	return 0
}

function test3()
{
	# create boost, list versions when no versions exist
	local bid=$($TURBO_HOME/bin/turbo-cli.exe boost add -q gzip )
	echo "Boost id is '$bid'"
	local vid=$($TURBO_HOME/bin/turbo-cli.exe version add -q $bid $(which gzip))
	echo "Version id is '$vid'"
	$TURBO_HOME/bin/turbo-cli.exe seed add $bid $TURBO_HOME/examples/gzip-seed.yaml
	$TURBO_HOME/bin/turbo-cli.exe fuzz --app-config $TURBO_HOME/examples/gzip-config.yaml --fuzz-config $TURBO_HOME/examples/afl.yaml --ver-id $vid
	$TURBO_HOME/bin/turbo-cli.exe log get prep $vid
	$TURBO_HOME/bin/turbo-cli.exe log get fuzz $vid

	return 0
}


function main()
{
	set -x
	set -e
	trap cleanup EXIT
	cd /tmp/turbo_test	
	. set_env_vars
	cd /tmp

	setup_server
	test1
	test2
	test3

	exit 0
}


main "$@"
