#ifndef option_h
#define option_h

#include <vector>
#include <string>

#define ALLOF(a) begin(a),end(a)

#define SETUP_CONN(a,b, opts)                                                \
        const auto opt_val = getOption(opts, "host");                        \
        const auto creds   = getCredentials(opts);                           \
        auto channel       = grpc::CreateChannel(opt_val,creds);             \
        auto backend       = TurboService::NewStub(channel);                 \
                                                                             \
        auto request = a();                                                  \
        auto reply   = b();                                                  \
        grpc::ClientContext context;



namespace Turbo
{


	using namespace std;
	struct Option_t
	{
		string name;
		string value;
	};

	using OptionVector_t = vector<Option_t>;
}
int Usage(int argc, char* argv[], Turbo::OptionVector_t& upper_opts);

namespace Turbo
{

	// typedef for a function pointer like main
	using MainLikeFuncPtr_t = int(*)(int argc, char *argv[], vector<Option_t>& upper_opts) ;

	struct SubCommand_t
	{
		string            name;
		MainLikeFuncPtr_t func;
	};

	using SubCommandVector_t = vector<SubCommand_t>;

	inline string getOption(const OptionVector_t& opts, const string& name)
	{

		const auto found_it = find_if(ALLOF(opts), [&](const Option_t& opt){ return opt.name==name; });
		assert(found_it!=end(opts));
		return found_it->value;

	}

	inline int32_t getOptionInt(const OptionVector_t& opts, const string& name)
	{
		return atoi(getOption(opts,name).c_str());
	}

	inline int subcommandDispatch(const string& cmd_msg, const SubCommandVector_t& cmds, int argc, char* argv[], OptionVector_t& upper_opts)
	{

		if(argc < 2)
		{
			cout << "Error:  must specify " << cmd_msg << " subcommand "<< endl;
			Usage(argc,argv, upper_opts);
			return 1;
		}

		const auto subcommand_it = find_if(ALLOF(cmds),
				[argv](const SubCommand_t& cmd) { return cmd.name == argv[1]; });
		if(subcommand_it ==  end(cmds))
		{
			cout << "Error:  Invalid " << cmd_msg << " subcommand:" << argv[1] << endl;
			Usage(argc,argv, upper_opts);
			return 1;
		}

		return subcommand_it->func(argc-1,argv+1, upper_opts);
	}

	inline shared_ptr<grpc::ChannelCredentials> getCredentials(const OptionVector_t& opts)
	{
		return grpc::InsecureChannelCredentials();
	}
	inline void print_error(const grpc::Status& status)
	{
		cout << "Server returned error. " << endl;
		cout << "Number:  " << status.error_code()    << endl;
		cout << "Message: " << status.error_message() << endl;
		cout << "Details: " << status.error_details() << endl;
	}


}

#endif
