
#include "fuzz-base.hpp"
#include "afl-fuzz.hpp"

using namespace Turbo;

unique_ptr<FuzzBase_t> FuzzBase_t::factory(const YAML::Node& _fuzz_config, const YAML::Node& _app_config, const OptionVector_t& _options, int32_t _ver_id)
{
	return make_unique<AflFuzz_t>(_fuzz_config,_app_config,_options,_ver_id);
}


int32_t FuzzBase_t::getAllInputs()
{

	SETUP_CONN(DownloadInputsRequest_t,DownloadInputsReply_t, options); 

	request.set_ver_id(ver_id);

        auto reader = move(unique_ptr<ClientReader<DownloadInputsReply_t> >( backend->DownloadInputs(&context, request)));


        while(reader->Read(&reply))
        {
		cout << "Server returned input of size " << dec << reply.yaml_string().size() << endl;
		inputs.push_back({reply.input_id(),YAML::Load(reply.yaml_string())});
        }

        const auto status = reader->Finish();

        // Act upon its status.
        if (!status.ok())
        {
                print_error(status);
                return 1;
        }
        cout << "Server returned " << dec << inputs.size() << " inputs for version " << ver_id << endl;
        return 0;


}
