#ifndef fuzzbase_hpp
#define fuzzbase_hpp

// std c/c++ includes
#include <iostream>
#include <fstream>
#include <sstream>
#include <memory>
#include <string>
#include <iomanip>
#include <cstdlib>
#include <unistd.h>
#include <getopt.h>

// grpc includes
#include <grpc++/grpc++.h>
#include "turbo.grpc.pb.h"

// yaml includes
#include <yaml-cpp/yaml.h>

// local includes
#include "option.hpp"

namespace Turbo
{
	using namespace std;
	using namespace grpc;

	class FuzzBase_t
	{
		public:
			// delete some default methods
			FuzzBase_t() = delete;
			FuzzBase_t(const FuzzBase_t& fb) = delete;

			// provided methods
			virtual ~FuzzBase_t() {}

			// virtual methods
			virtual int32_t execute() = 0;

			// factories
			static unique_ptr<FuzzBase_t> factory(const YAML::Node& _fuzz_config, const YAML::Node& _app_config, const OptionVector_t& options, const int32_t ver_id);


		protected:

			// only children can construct me.
			FuzzBase_t(const OptionVector_t &_options, const int32_t _ver_id) : options(_options), ver_id(_ver_id) {}

			virtual int32_t getAllInputs(); 


			// shareable data

			const OptionVector_t options;
			const int32_t ver_id;
			using InputData_t = struct 
			{
				uint32_t id;
				YAML::Node yaml;
			};

			vector<InputData_t> inputs;
		private:
	};

}
#endif
